#!/bin/sh

LOGOUT=/dev/stdout
export PATH=/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin

. ./update_check_functions
. ./updater-functions

export TIME=time
BOOT="UPDATE"
USED="user"
UPDATE_MP=$1
MODEL=
TAR_FILE=${TMP}/update.Sony.Reader.tar
PACKAGE_INFO=${TMP}/SETTING
HEADER_INFO=${TMP}/HEADER
ENC_TAR=${TAR_FILE}.enc

##############################
##############################
# arg
#   $1 : UPDATE_IV (hex string)
#   $2 : File Size (decimal)
#   $3 : Checksum
#   $4 : FName
make_header()
{
   /bin/echo -en `echo $1 | sed 's/../\\\\x&/g'` > $4
   /bin/echo -en `printf "%08x" $2 | sed 's/../\\\\x&/g'` >> $4
   printf "%-12s" $3 >> $4
}

##############################
# arg
#   $1 : Package TAR file path
#   $2 : SETTING file path
#   $3 : Full package
generate_update ()
{
	echo "### GEN UPDATER : $1 -> $3"

	# CHECKSUM=`cksum $1 | awk '{ print $1 }'`
	CHECKSUM=`cat $1 | cksum`
	UPDATE_IV=000102030405060708090a0b0c0d0e0f
	SIGNATURE=`echo \`cat $1 | openssl sha1 -binary | openssl rsautl -sign -inkey ${SIG_KEY} | hexdump -n 512 -v -C | head -n 32 | awk '{ print $2$3$4$5$6$7$8$9$10$11$12$13$14$15$16$17 }'\` | sed 's/ //g'`
	OFFSET=0

	# openssl aes-128-cbc -e -K ${ENC_KEY} -iv ${UPDATE_IV} -in $1 | dd if=/dev/stdin of=${TMP}/${1}.enc bs=1k conv=sync
	openssl aes-128-cbc -e -K ${ENC_KEY} -iv ${UPDATE_IV} -in $1 > ${ENC_TAR}

	BLOCK=`ls -lah ${ENC_TAR} | awk '{ print $5 }'`
	SIZE=`ls -la ${ENC_TAR} | awk '{ print $5 }'`
	LINE="UPDATER $OFFSET $BLOCK $CHECKSUM $UPDATE_IV $SIGNATURE"
	echo "# MODEL " $MODEL > $2
	echo "# SKU " $SKU >> $2
	echo $LINE >> $2

	
	H_UPDATE_IV=$UPDATE_IV
	H_SIZE=`ls -la $2 | awk '{ print $5 }'`
	H_CHECKSUM=`cksum $2 | awk '{ print $1 }'`

	if [ "${TIME}" = "time" ];
	then
		echo "  H_UPDATE_IV       ${H_UPDATE_IV}"
		echo "  H_SIZE            ${H_SIZE}"
		echo "  H_CHECKSUM        ${H_CHECKSUM}"
	fi

	PACKAGE_HEADER_SIZE=$(((H_SIZE + 64 + 1023) / 1024))
	OFFSET=$((OFFSET + PACKAGE_HEADER_SIZE))

	# ./mkhdr $H_UPDATE_IV $H_SIZE $H_CHECKSUM | openssl des -e -k ${ENC_KEY} | dd if=/dev/stdin of=$HEADER_INFO bs=64 count=1 conv=sync
	make_header $H_UPDATE_IV $H_SIZE $H_CHECKSUM $STDOUT | openssl des -e -k ${ENC_KEY} | dd if=/dev/stdin of=$HEADER_INFO bs=64 count=1 conv=sync
	openssl aes-128-cbc -e -K ${ENC_KEY} -iv ${UPDATE_IV} -in $2 >> $HEADER_INFO
	dd if=$HEADER_INFO of="$3" bs=1K conv=sync
	cat ${ENC_TAR} >> $3

	
	
	LINE=`grep -e '^UPDATER' $2`
	OFFSET=`echo "${LINE}" | awk '{ print $2 }'`
	BLOCK=`echo "${LINE}" | awk '{ print $3 }'`
	CHECKSUM=`echo "${LINE}" | awk '{ print $4" "$5 }'`
	SIZE=$((((`echo "${LINE}" | awk '{ print $5 }'` + 16) / 16) * 16))
	UPDATE_IV=`echo "${LINE}" | awk '{ print $6 }'`
	SIGNATURE=`echo "${LINE}" | awk '{ print $7 }'`

	if [ "${TIME}" = "time" ];
	then
		echo "  LINE              ${LINE}"
		echo "  OFFSET            ${OFFSET}"
		echo "  BLOCK             ${BLOCK}"
		echo "  CHECKSUM          ${CHECKSUM}"
		echo "  SIZE              ${SIZE}"
		echo "  UPDATE_IV         ${UPDATE_IV}"
		echo "  SIGNATURE         ${SIGNATURE}"
	fi
}

if [ $# -ne 1 -o ! -d $1 ]
then
  echo "usage: $0 <upd_dir>"
  exit 1
fi

if [ ! -x ${1}/update.sh ];
then
   echo  update.sh not found in directory
   exit 1
fi

rm -rf ${1}/*Updater.package

get_info_key

export PACKAGE="${UPDATE_MP}/${MODEL} Updater.package"

rm -rf ${PACKAGE}
tar -cvf $TAR_FILE -C ${UPDATE_MP} .

if [ ! -f $TAR_FILE ];
then
   echo Update file not found
   exit 2
fi

generate_update "$TAR_FILE" "$PACKAGE_INFO" "$PACKAGE"

